import type { Source, Target, Node, Bound, Entry } from "@canvas-js/okra";
import { TypedEventEmitter } from "@libp2p/interface";
import { Logger } from "@libp2p/logger";
import type { Signature, Signer, Message, Awaitable } from "@canvas-js/interfaces";
import { Mempool } from "./Mempool.js";
import { DelayableController } from "./utils.js";
export interface ReadOnlyTransaction {
    getHeads: () => Awaitable<Uint8Array[]>;
    getAncestors: (key: Uint8Array, atOrBefore: number, results: Set<string>) => Awaitable<void>;
    isAncestor: (key: Uint8Array, ancestorKey: Uint8Array, visited?: Set<string>) => Awaitable<boolean>;
    messages: Target;
}
export interface ReadWriteTransaction extends ReadOnlyTransaction {
    insert: (id: string, signature: Signature, message: Message, entry?: Entry) => Awaitable<void>;
}
export type GossipLogConsumer<Payload = unknown> = (id: string, signature: Signature, message: Message<Payload>) => Awaitable<void>;
export interface GossipLogInit<Payload = unknown> {
    topic: string;
    apply: GossipLogConsumer<Payload>;
    validatePayload?: (payload: unknown) => payload is Payload;
    verifySignature?: (signature: Signature, message: Message<Payload>) => Awaitable<void>;
    signer?: Pick<Signer<Payload>, "sign" | "verify">;
    indexAncestors?: boolean;
}
export type GossipLogEvents<Payload = unknown> = {
    message: CustomEvent<{
        id: string;
        signature: Signature;
        message: Message<Payload>;
    }>;
    commit: CustomEvent<{
        root: Node;
    }>;
    sync: CustomEvent<{
        peer?: string;
        duration: number;
        messageCount: number;
    }>;
    error: CustomEvent<{
        error: Error;
    }>;
};
export declare abstract class AbstractGossipLog<Payload = unknown> extends TypedEventEmitter<GossipLogEvents<Payload>> {
    #private;
    abstract close(): Promise<void>;
    protected abstract entries(lowerBound?: Bound<Uint8Array> | null, upperBound?: Bound<Uint8Array> | null, options?: {
        reverse?: boolean;
    }): AsyncIterable<[key: Uint8Array, value: Uint8Array]>;
    protected abstract read<T>(callback: (txn: ReadOnlyTransaction) => Awaitable<T>, options?: {
        targetId?: string;
    }): Promise<T>;
    protected abstract write<T>(callback: (txn: ReadWriteTransaction) => Awaitable<T>, options?: {
        sourceId?: string;
    }): Promise<T>;
    readonly topic: string;
    readonly indexAncestors: boolean;
    readonly signer: Pick<Signer<Payload>, "sign" | "verify">;
    protected readonly log: Logger;
    protected readonly mempool: Mempool<{
        signature: Signature;
        message: Message<Payload>;
    }>;
    protected constructor(init: GossipLogInit<Payload>);
    replay(): Promise<void>;
    iterate(lowerBound?: {
        id: string;
        inclusive: boolean;
    } | null, upperBound?: {
        id: string;
        inclusive: boolean;
    } | null, options?: {
        reverse?: boolean;
    }): AsyncIterable<[id: string, signature: Signature, message: Message<Payload>]>;
    encode(signature: Signature, message: Message<Payload>): [key: Uint8Array, value: Uint8Array];
    decode(value: Uint8Array): [id: string, signature: Signature, message: Message<Payload>];
    getClock(): Promise<[clock: number, heads: string[]]>;
    has(id: string): Promise<boolean>;
    get(id: string): Promise<[signature: Signature, message: Message<Payload>] | [null, null]>;
    /**
     * Sign and append a new *unsigned* message to the end of the log.
     * The currently unmerged heads of the local log are used as parents.
     */
    append(payload: Payload, options?: {
        signer?: Pick<Signer<Payload>, "sign" | "verify">;
    }): Promise<{
        id: string;
        signature: Signature;
        message: Message<Payload>;
    }>;
    /**
     * Insert an existing signed message into the log (ie received via PubSub).
     * If any of the parents are not present, insert the message into the mempool instead.
     */
    insert(signature: Signature, message: Message<Payload>): Promise<{
        id: string;
    }>;
    getAncestors(id: string, atOrBefore: number): Promise<string[]>;
    isAncestor(id: string, ancestor: string): Promise<boolean>;
    /**
     * Sync with a remote source, applying and inserting all missing messages into the local log
     */
    sync(source: Source, options?: {
        sourceId?: string;
        timeoutController?: DelayableController;
    }): Promise<{
        root: Node;
        messageCount: number;
    }>;
    /**
     * Serve a read-only snapshot of the merkle tree
     */
    serve(callback: (source: Source) => Promise<void>, options?: {
        targetId?: string;
    }): Promise<void>;
}
